@@ -1,20 +1,21 @@
 UFA_enumerated_chemical_space_xlsxAnalyzer <- function(PARAM_MF) {
+  ##
   tr1_sub_pattern <- Sys.time()
   ##
-  checkpoint_parameter <- 1
+  checkpoint_parameter <- TRUE
   ##
   x_address_IPDB <- which(PARAM_MF$Parameter == "IPDB output address")
   address_IPDB <- PARAM_MF$`User input 2`[x_address_IPDB]
   address_IPDB <- gsub("\\", "/", address_IPDB, fixed = TRUE)
   if (!dir.exists(address_IPDB)) {
-    checkpoint_parameter <- 0
+    checkpoint_parameter <- FALSE
     print("ERROR!!! Please make sure the full path is provided for the 'IPDB output address' in the 'enumerated_chemical_space' tab!")
   }
   ##
   x_peak_spacing <- which(PARAM_MF$Parameter == "Peak spacing (Da)")
   peak_spacing <- as.numeric(PARAM_MF$`User input 2`[x_peak_spacing])
   if (is.na(peak_spacing)) {
-    checkpoint_parameter <- 0
+    checkpoint_parameter <- FALSE
     print("Error!!! Peak Spacing in the 'enumerated_chemical_space' tab should be a positive number!")
   }
   ##
@@ -22,14 +23,14 @@ UFA_enumerated_chemical_space_xlsxAnalyzer <- function(PARAM_MF) {
   IP_mem_usage <- PARAM_MF$`User input 2`[x_IP_mem_usage]
   UFA_IP_memeory_variables <- eval(parse(text = IP_mem_usage))
   if (length(UFA_IP_memeory_variables) != 3) {
-    checkpoint_parameter <- 0
+    checkpoint_parameter <- FALSE
     print("Error!!! Isotopic profile calculations memory usage in the 'enumerated_chemical_space' tab should be a vector of three positive numbers!")
   }
   ##
   x_npc <- which(PARAM_MF$Parameter == "Number of parallel threads")
   number_processing_threads <- as.numeric(PARAM_MF$`User input 2`[x_npc])
   if (is.na(number_processing_threads)) {
-    checkpoint_parameter <- 0
+    checkpoint_parameter <- FALSE
     print("Error!!! Number of parallel threads in the 'enumerated_chemical_space' tab should be an integer!")
   }
   ##
@@ -37,12 +38,12 @@ UFA_enumerated_chemical_space_xlsxAnalyzer <- function(PARAM_MF) {
   LOWEST_mass <- as.numeric(PARAM_MF$`User input 1`[mr_x])
   HIGHEST_mass <- as.numeric(PARAM_MF$`User input 2`[mr_x])
   if (is.na(LOWEST_mass) | is.na(HIGHEST_mass)) {
-    checkpoint_parameter <- 0
+    checkpoint_parameter <- FALSE
     print("Error!!! Check 'Mass range' in the 'enumerated_chemical_space' tab!")
   } else {
     if (!is.na(LOWEST_mass) & !is.na(HIGHEST_mass)) {
       if (LOWEST_mass > HIGHEST_mass) {
-        checkpoint_parameter <- 0
+        checkpoint_parameter <- FALSE
         print("Error!!! Check 'Mass range' in the 'enumerated_chemical_space' tab!")
       }
     }
@@ -52,12 +53,12 @@ UFA_enumerated_chemical_space_xlsxAnalyzer <- function(PARAM_MF) {
   c_xyz1 <- as.numeric(PARAM_MF$`User input 1`[c_x])
   c_xyz2 <- as.numeric(PARAM_MF$`User input 2`[c_x])
   if (is.na(c_xyz1) | is.na(c_xyz2)) {
-    checkpoint_parameter <- 0
+    checkpoint_parameter <- FALSE
     print("Error!!! Check the 'Carbon' range in the 'enumerated_chemical_space' tab!")
   } else {
     if (!is.na(c_xyz1) & !is.na(c_xyz2)) {
       if (c_xyz1 > c_xyz2) {
-        checkpoint_parameter <- 0
+        checkpoint_parameter <- FALSE
         print("Error!!! Check the 'Carbon' range in the 'enumerated_chemical_space' tab!")
       }
     }
@@ -67,7 +68,8 @@ UFA_enumerated_chemical_space_xlsxAnalyzer <- function(PARAM_MF) {
     do.call(rbind, lapply((b_xyz1):(b_xyz2), function(b) { # Boron range
       do.call(rbind, lapply((br_xyz1):(br_xyz2), function(br) { # Bromine range
         do.call(rbind, lapply((cl_1):(cl_2), function(cl) { # Chlorine range
-          if (((br + cl) >= (sum_br_cl_xyz_1)) & ((br + cl) <= (sum_br_cl_xyz_2))) { # Optional condition to remove low probable compounds
+          SUMbrcl <- br + cl
+          if ((SUMbrcl >= (sum_br_cl_xyz_1)) & (SUMbrcl <= (sum_br_cl_xyz_2))) { # Optional condition to remove low probable compounds
             do.call(rbind, lapply((k_1):(k_2), function(k) { # Potassium range
               do.call(rbind, lapply((s_1):(s_2), function(s) { # Sulfur range
                 do.call(rbind, lapply((se_1):(se_2), function(se) { # Selenium range
@@ -103,7 +105,8 @@ UFA_enumerated_chemical_space_xlsxAnalyzer <- function(PARAM_MF) {
   Ess_MolVecMat_call <- gsub("cl_1", cl_1, Ess_MolVecMat_call)
   Ess_MolVecMat_call <- gsub("cl_2", cl_2, Ess_MolVecMat_call)
   #
-  sum_br_cl_xyz_x <- which(PARAM_MF$Parameter == "Sum Br + Cl")
+  sum_Rule3_x <- grep("Rule 3", PARAM_MF$Parameter, ignore.case = TRUE)
+  sum_br_cl_xyz_x <- sum_Rule3_x[1]
   sum_br_cl_xyz_1 <- as.numeric(PARAM_MF$`User input 1`[sum_br_cl_xyz_x])
   sum_br_cl_xyz_2 <- as.numeric(PARAM_MF$`User input 2`[sum_br_cl_xyz_x])
   Ess_MolVecMat_call <- gsub("sum_br_cl_xyz_1", sum_br_cl_xyz_1, Ess_MolVecMat_call)
@@ -154,29 +157,47 @@ UFA_enumerated_chemical_space_xlsxAnalyzer <- function(PARAM_MF) {
     s <- Ess_MolVecMat[counter, 6]
     se <- Ess_MolVecMat[counter, 7]
     si <- Ess_MolVecMat[counter, 8]
+    ##
     do.call(rbind, lapply((n_1):(n_2), function(n) { # Nitrogen range
       do.call(rbind, lapply((h_1):(h_2), function(h) { # Hydrogen range
         do.call(rbind, lapply((as_xyz1):(as_xyz2), function(as) { # Arsenic range
           do.call(rbind, lapply((f_xyz1):(f_xyz2), function(f) { # Fluorine range
             do.call(rbind, lapply((i_xyz1):(i_xyz2), function(i) { # Iodine range
-              if (((br + cl + f + i) >= (sum_br_cl_f_i_1)) & ((br + cl + f + i) <= (sum_br_cl_f_i_2))) { # Optional condition to remove low probable compounds
-                do.call(rbind, lapply((na_1):(na_2), function(na) { # Sodium range
-                  do.call(rbind, lapply((o_1):(o_2), function(o) { # Oxygen range
-                    do.call(rbind, lapply((p_1):(p_2), function(p) { # Phosphorus range
-                      if ((COND2)) {
-                        mol_vec <- c(c, h, as, b, br, cl, f, i, k, n, na, o, p, s, se, si)
-                        if (extended_SENIOR_rule_str) {
-                          rule2 <- extended_SENIOR_rule_check(mol_vec, valence_vec, ionization_correction = ipw_n)
-                        } else {
-                          rule2 <- TRUE
+              ##
+              SUMbrclfi <- br + cl + f + i
+              if ((SUMbrclfi >= (sum_br_cl_f_i_1)) & (SUMbrclfi <= (sum_br_cl_f_i_2))) { # Optional condition to remove low probable compounds
+                ##
+                if (rule1Check) {
+                  SUMhbrclfi <- SUMbrclfi + h
+                  rule1 <- ifelse((SUMhbrclfi >= (c/2 - n - 1) & SUMhbrclfi <= (2*c + 3*n + 6)), TRUE, FALSE)
+                } else {
+                  rule1 <- TRUE
+                }
+                if (rule1) {
+                  ##
+                  do.call(rbind, lapply((na_1):(na_2), function(na) { # Sodium range
+                    do.call(rbind, lapply((o_1):(o_2), function(o) { # Oxygen range
+                      do.call(rbind, lapply((p_1):(p_2), function(p) { # Phosphorus range
+                        if ((COND2)) {
+                          ##
+                          NUM14elements <- length(which(c(c, h, as, b, br, cl, f, i, n, o, p, s, se, si) > 0)) ## Na and K were mot included in this equation
+                          if (NUM14elements <= maxNUM14elements) {
+                            ##
+                            mol_vec <- c(c, h, as, b, br, cl, f, i, k, n, na, o, p, s, se, si)
+                            if (extended_SENIOR_rule_str) {
+                              rule2 <- extended_SENIOR_rule_check(mol_vec, valence_vec, ionization_correction = ipw_n)
+                            } else {
+                              rule2 <- TRUE
+                            }
+                            if (rule2) {
+                              c(mol_vec, counter)
+                            }
+                          }
                         }
-                        if (rule2) {
-                          c(mol_vec, counter)
-                        }
-                      }
+                      }))
                     }))
                   }))
-                }))
+                }
               }
             }))
           }))
@@ -215,7 +236,7 @@ UFA_enumerated_chemical_space_xlsxAnalyzer <- function(PARAM_MF) {
   MolVecMat_call <- gsub("i_xyz1", i_xyz1, MolVecMat_call)
   MolVecMat_call <- gsub("i_xyz2", i_xyz2, MolVecMat_call)
   #
-  sum_br_cl_f_i_x <- which(PARAM_MF$Parameter == "Sum Br + Cl + F + I")
+  sum_br_cl_f_i_x <- sum_Rule3_x[2]
   sum_br_cl_f_i_1 <- PARAM_MF$`User input 1`[sum_br_cl_f_i_x]
   sum_br_cl_f_i_2 <- PARAM_MF$`User input 2`[sum_br_cl_f_i_x]
   MolVecMat_call <- gsub("sum_br_cl_f_i_1", sum_br_cl_f_i_1, MolVecMat_call)
@@ -239,14 +260,16 @@ UFA_enumerated_chemical_space_xlsxAnalyzer <- function(PARAM_MF) {
   MolVecMat_call <- gsub("p_1", p_1, MolVecMat_call)
   MolVecMat_call <- gsub("p_2", p_2, MolVecMat_call)
   #
-  COND2_x <- which(PARAM_MF$Parameter == "Condition2")
-  COND2 <- PARAM_MF$`User input 2`[COND2_x]
-  if (COND2 == "" | COND2 == " " | COND2 == "1" | tolower(COND2) == "t" | tolower(COND2) == "true") {
-    COND2 <- "TRUE"
+  x_rule1 <- grep("Rule 1", PARAM_MF$Parameter, ignore.case = TRUE)
+  str_rule1 <- PARAM_MF$`User input 2`[x_rule1]
+  if (str_rule1 == "1" | tolower(str_rule1) == "t" | tolower(str_rule1) == "true") {
+    rule1Check <- "TRUE"
+  } else {
+    rule1Check <- "FALSE"
   }
-  MolVecMat_call <- gsub("COND2", COND2, MolVecMat_call)
+  MolVecMat_call <- gsub("rule1Check", rule1Check, MolVecMat_call)
   #
-  ext_sen_x <- which(PARAM_MF$Parameter == "Extended SENIOR rule")
+  ext_sen_x <- grep("Extended SENIOR rule", PARAM_MF$Parameter, ignore.case = TRUE)
   ext_sen <- PARAM_MF$`User input 2`[ext_sen_x]
   if (ext_sen == "1" | tolower(ext_sen) == "t" | tolower(ext_sen) == "true") {
     extended_SENIOR_rule_str <- "TRUE"
@@ -262,68 +285,101 @@ UFA_enumerated_chemical_space_xlsxAnalyzer <- function(PARAM_MF) {
       ipw_n <- "-1"
     } else if (ipw == "[M-H]") {
       ipw_n <- "+1"
-    } else if (ipw == "[M]") {
-      ipw_n <- "0"
     } else {
-      checkpoint_parameter <- 0
-      print("Error!!! Check 'Ionization pathway' in the 'enumerated_chemical_space' tab!")
       ipw_n <- "0"
     }
     MolVecMat_call <- gsub("ipw_n", ipw_n, MolVecMat_call)
   }
+  #
+  maxNUM14elements <- as.numeric(PARAM_MF$`User input 2`[grep("Rule 4", PARAM_MF$Parameter, ignore.case = TRUE)])
+  if (is.na(maxNUM14elements)) {
+    checkpoint_parameter <- FALSE
+    print("Error!!! Check 'Rule 4 (Maximum number of elements rule)'!")
+  } else {
+    if (maxNUM14elements > 14 & maxNUM14elements < 1) {
+      checkpoint_parameter <- FALSE
+      print("Error!!! Check 'Rule 4 (Maximum number of elements rule)'!")
+    }
+  }
+  #
+  COND2_x <- which(PARAM_MF$Parameter == "Condition2")
+  COND2 <- PARAM_MF$`User input 2`[COND2_x]
+  if (COND2 == "" | COND2 == " " | COND2 == "1" | tolower(COND2) == "t" | tolower(COND2) == "true") {
+    COND2 <- "TRUE"
+  }
+  MolVecMat_call <- gsub("COND2", COND2, MolVecMat_call)
   ##############################################################################
   eval(parse(text = MolVecMat_call))
   ##
-  ElementsAlphabetical <- c("C", "H", "As", "B", "Br", "Cl", "F", "I", "K", "N", "Na", "O", "P", "S", "Se", "Si")
-  L_ElementsAlphabetical <- length(ElementsAlphabetical)
-  if (extended_SENIOR_rule_str == "TRUE") {
-    EL_Alphabetical <- element_sorter(ElementList = ElementsAlphabetical, ElementOrder = "same")
-    valence_vec <- EL_Alphabetical[[3]]
-  }
-  ##############################################################################
-  osType <- Sys.info()[['sysname']]
-  if (osType == "Windows") {
-    clust <- makeCluster(number_processing_threads)
-    registerDoParallel(clust)
+  if (checkpoint_parameter) {
+    ElementsAlphabetical <- c("C", "H", "As", "B", "Br", "Cl", "F", "I", "K", "N", "Na", "O", "P", "S", "Se", "Si")
+    L_ElementsAlphabetical <- length(ElementsAlphabetical)
+    if (extended_SENIOR_rule_str == "TRUE") {
+      EL_Alphabetical <- element_sorter(ElementList = ElementsAlphabetical, ElementOrder = "same")
+      valence_vec <- EL_Alphabetical[["Valence"]]
+    }
+    ##
+    ############################################################################
     ##
     print("Initiated counting essential elements combinations!")
-    Ess_MolVecMat <- foreach(c = c_xyz1:c_xyz2, .combine = 'rbind', .verbose = FALSE) %dopar% {
-      Ess_MolVecMat_call(c)
-    }
-    Ess_MolVecMat <- matrix(Ess_MolVecMat, ncol = L_Essential_Elements)
-    L_Ess_MolVecMat <- dim(Ess_MolVecMat)[1]
-    print(paste0("There are ", L_Ess_MolVecMat, " essential elements combinations!"))
     ##
-    print("Initiated enumerating molecular formulas!")
-    MolVecMat <- foreach(counter = 1:L_Ess_MolVecMat, .verbose = FALSE) %dopar% {
-      MolVecMat_call(counter)
+    if (number_processing_threads == 1) {
+      ##
+      Ess_MolVecMat <- do.call(rbind, lapply(c_xyz1:c_xyz2, function(c) {
+        Ess_MolVecMat_call(c)
+      }))
+      Ess_MolVecMat <- matrix(Ess_MolVecMat, ncol = L_Essential_Elements)
+      L_Ess_MolVecMat <- dim(Ess_MolVecMat)[1]
+      print(paste0("There are ", L_Ess_MolVecMat, " essential elements combinations!"))
+      ##
+      print("Initiated enumerating molecular formulas!")
+      MolVecMat <- do.call(rbind, lapply(1:L_Ess_MolVecMat, function(counter) {
+        MolVecMat_call(counter)
+      }))
+      ##
+    } else {
+      ##
+      osType <- Sys.info()[['sysname']]
+      if (osType == "Linux") {
+        ##
+        Ess_MolVecMat <- do.call(rbind, mclapply(c_xyz1:c_xyz2, function(c) {
+          Ess_MolVecMat_call(c)
+        }, mc.cores = number_processing_threads))
+        Ess_MolVecMat <- matrix(Ess_MolVecMat, ncol = L_Essential_Elements)
+        L_Ess_MolVecMat <- dim(Ess_MolVecMat)[1]
+        print(paste0("There are ", L_Ess_MolVecMat, " essential elements combinations!"))
+        ##
+        print("Initiated enumerating molecular formulas!")
+        MolVecMat <- do.call(rbind, mclapply(1:L_Ess_MolVecMat, function(counter) {
+          MolVecMat_call(counter)
+        }, mc.cores = number_processing_threads))
+        ##
+        closeAllConnections()
+        ##
+      } else if (osType == "Windows") {
+        clust <- makeCluster(number_processing_threads)
+        registerDoParallel(clust)
+        ##
+        Ess_MolVecMat <- foreach(c = c_xyz1:c_xyz2, .combine = 'rbind', .verbose = FALSE) %dopar% {
+          Ess_MolVecMat_call(c)
+        }
+        Ess_MolVecMat <- matrix(Ess_MolVecMat, ncol = L_Essential_Elements)
+        L_Ess_MolVecMat <- dim(Ess_MolVecMat)[1]
+        print(paste0("There are ", L_Ess_MolVecMat, " essential elements combinations!"))
+        ##
+        print("Initiated enumerating molecular formulas!")
+        MolVecMat <- foreach(counter = 1:L_Ess_MolVecMat, .verbose = FALSE) %dopar% {
+          MolVecMat_call(counter)
+        }
+        ##
+        stopCluster(clust)
+        ##
+      }
     }
+    ##
     MolVecMat <- matrix(MolVecMat, ncol = (L_ElementsAlphabetical + 1))
     L_MolVecMat <- dim(MolVecMat)[1]
     ##
-    stopCluster(clust)
-    ##
-  } else if (osType == "Linux") {
-    ##
-    print("Initiated counting essential elements combinations!")
-    Ess_MolVecMat <- do.call(rbind, mclapply(c_xyz1:c_xyz2, function(c) {
-      Ess_MolVecMat_call(c)
-    }, mc.cores = number_processing_threads))
-    Ess_MolVecMat <- matrix(Ess_MolVecMat, ncol = L_Essential_Elements)
-    L_Ess_MolVecMat <- dim(Ess_MolVecMat)[1]
-    print(paste0("There are ", L_Ess_MolVecMat, " essential elements combinations!"))
-    ##
-    print("Initiated enumerating molecular formulas!")
-    MolVecMat <- do.call(rbind, mclapply(1:L_Ess_MolVecMat, function(counter) {
-      MolVecMat_call(counter)
-    }, mc.cores = number_processing_threads))
-    MolVecMat <- matrix(MolVecMat, ncol = (L_ElementsAlphabetical + 1))
-    L_MolVecMat <- dim(MolVecMat)[1]
-    ##
-    closeAllConnections()
-  }
-  ##
-  if (checkpoint_parameter == 1) {
     print(paste0("The approximate maximum number of the candidate molecular formulas is ", L_MolVecMat, " !"))
     ##
     print("The required time only to iterate loops through this chemical space is :")
